@@ -9,6 +9,7 @@ pub use switchboard_protos::protos::aggregator_state::RoundResult;
 pub use switchboard_protos::protos::switchboard_account_types::SwitchboardAccountType;
 use solana_program::pubkey::Pubkey;
 use zerocopy::AsBytes;
+use switchboard_protos::protos::vrf::VrfAccountData;
 
 /// Returns whether the current open round is considered valid for usage.
 pub fn is_current_round_valid(aggregator: &AggregatorState) -> Result<bool, ProgramError> {
@@ -30,6 +31,9 @@ pub fn is_current_round_valid(aggregator: &AggregatorState) -> Result<bool, Prog
 /// account is not initialized as an aggregator.
 pub fn get_aggregator(switchboard_feed: &AccountInfo) -> Result<AggregatorState, ProgramError> {
     let state_buffer = switchboard_feed.try_borrow_data()?;
+    if state_buffer.len() == 0 || state_buffer[0] != SwitchboardAccountType::TYPE_AGGREGATOR as u8 {
+        return Err(ProgramError::InvalidAccountData);
+    }
     let aggregator_state: AggregatorState =
         deserialize_from_slice(&state_buffer[1..]).map_err(|_| ProgramError::InvalidAccountData)?;
     Ok(aggregator_state)
@@ -80,6 +84,44 @@ impl<'a> BundleAccount<'a> {
     }
 }
 
+
+
+pub struct VrfAccount<'a> {
+    account_info: &'a AccountInfo<'a>
+}
+
+impl<'a> VrfAccount<'a> {
+
+    pub fn new(account: &'a AccountInfo<'a>) -> Result<Self, ProgramError> {
+        let buf = account.try_borrow_data()?;
+        if buf.len() == 0 || buf[0] != SwitchboardAccountType::TYPE_VRF as u8 {
+            return Err(ProgramError::InvalidAccountData);
+        }
+        Ok(Self{
+            account_info: account
+        })
+    }
+
+    /// returns the current verified randomness value held in the account.
+    /// returns ProgramError if not randomness currently exists or
+    /// if the number of proof verificaitons is less than the reuired
+    /// minimum numner of verifications.
+    pub fn get_verified_randomness(&self) -> Result<Vec<u8>, ProgramError> {
+        let vrf_state: VrfAccountData =
+            deserialize_from_slice(&self.account_info.try_borrow_data()?[1..])
+                .map_err(|_| ProgramError::InvalidAccountData)?;
+        let value = vrf_state.value.ok_or(ProgramError::InvalidAccountData)?;
+        let min_confirmations = vrf_state.min_proof_confirmations
+            .ok_or(ProgramError::InvalidAccountData)?;
+        let num_confirmations = vrf_state.num_proof_confirmations
+            .ok_or(ProgramError::InvalidAccountData)?;
+        if num_confirmations < min_confirmations {
+            return Err(ProgramError::InvalidAccountData);
+        }
+        Ok(value)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
